﻿using ConvictButWithBigHonkers;
using ItemAPI;
using Mod;
using MonoMod.RuntimeDetour;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using UnityEngine;

namespace HunterPirateMod
{
    public class HunterModule : ETGModule
    {
        public static string ZipFilePath;
        public static string FilePath;
        public static readonly string MOD_NAME = "Hunter's Pirate Mod";
        public static readonly string VERSION = "0.0.0";
        public static readonly string TEXT_COLOR = "#00FFFF";

        public override void Start()
        {
            ZipFilePath = this.Metadata.Archive;
            FilePath = this.Metadata.Directory;
            AudioResourceLoader.InitAudio();
            ItemBuilder.Init();
            PirateShip.Add();
            SynergyInit();
            Log($"{MOD_NAME} v{VERSION} started successfully.", TEXT_COLOR);
        }

        private void SynergyInit()
        {
            List<string> mandatoryConsoleID1 = new List<string> {
            "hunter:captain_flintlock"
            };
            List<string> optionalConsoleID1 = new List<string> {
            "teapot",
            "ballot",
            "crown_of_guns",
            "double_vision"
            };
            CustomSynergies.Add("Sons Of Liberty", mandatoryConsoleID1, optionalConsoleID1);

            List<string> mandatoryConsoleID2 = new List<string> {
            "hunter:captain_flintlock"
            };
            List<string> optionalConsoleID2 = new List<string> {
            "corsair",
            "dueling_pistol",
            "blunderbuss",
            "serious_cannon"
            };
            CustomSynergies.Add("Cannon Crew", mandatoryConsoleID2, optionalConsoleID2);

            List<string> mandatoryConsoleID3 = new List<string> {
            "hunter:captain_flintlock"
            };
            List<string> optionalConsoleID3 = new List<string> {
            "abyssal_tentacle",
            "high_kaliber",
            "yellow_chamber",
            "cursed_bullets",
            "holey_grail"
            };
            CustomSynergies.Add("Kaliber's Trench", mandatoryConsoleID3, optionalConsoleID3);
        }

        public static void Log(string text, string color="#FFFFFF")
        {
            ETGModConsole.Log($"<color={color}>{text}</color>");
        }



        // this hook adds an event that tiggers every time any gameactor (players+enemies mostly) comes into contact with any goop, returing the DeadlyDeadlyGoopManager for that type of goop (mostly for identification purposes), the gameactor and the positing where the contact happened
        Hook GoopEffect = new Hook(typeof(DeadlyDeadlyGoopManager).GetMethod("DoTimelessGoopEffect", BindingFlags.Instance | BindingFlags.Public), typeof(HunterModule).GetMethod("ApplyGoopEffect"));
        public static Action<DeadlyDeadlyGoopManager, GameActor, IntVector2> OnGoopTouched;
        public static void ApplyGoopEffect(Action<DeadlyDeadlyGoopManager, GameActor, IntVector2> orig, DeadlyDeadlyGoopManager self, GameActor actor, IntVector2 goopPosition)
        {
            orig(self, actor, goopPosition);

            Action<DeadlyDeadlyGoopManager, GameActor, IntVector2> touchgoop = HunterModule.OnGoopTouched;
            if (touchgoop != null && self != null && actor != null && goopPosition != null)
            {
                touchgoop(self, actor, goopPosition);
            }
            return;
        }

        public override void Exit() { }
        public override void Init() { }
    }
    public static class SynergyChecker
    {
        // Token: 0x06000339 RID: 825 RVA: 0x0001E88C File Offset: 0x0001CA8C
        public static bool PlayerHasActiveSynergy(this PlayerController player, string synergyNameToCheck)
        {
            foreach (int num in player.ActiveExtraSynergies)
            {
                AdvancedSynergyEntry advancedSynergyEntry = GameManager.Instance.SynergyManager.synergies[num];
                bool flag = advancedSynergyEntry.NameKey == synergyNameToCheck;
                if (flag)
                {
                    return true;
                }
            }
            return false;
        }
    }


}
