﻿using System;
using System.Collections;
using System.Collections.Generic;
using Dungeonator;
using Gungeon;
using ItemAPI;
using UnityEngine;
using HunterCustomGungeonAPI;
using HunterPirateMod;

namespace Mod
{
	public class PirateShip : GunBehaviour
	{
		public static void Add()
		{
			Gun gun = ETGMod.Databases.Items.NewGun("Captain Flintlock", "parrot");
			Game.Items.Rename("outdated_gun_mods:captain_flintlock", "hunter:captain_flintlock");
			gun.gameObject.AddComponent<PirateShip>();
			GunExt.SetShortDescription(gun, "Terror Of The Seas");
			GunExt.SetLongDescription(gun, "This parrot has seen things...");
			GunExt.SetupSprite(gun, null, "parrot_idle_001", 0);
			GunExt.SetAnimationFPS(gun, gun.shootAnimation, 0);
			GunExt.SetAnimationFPS(gun, gun.reloadAnimation, 0);
			GunExt.SetAnimationFPS(gun, gun.idleAnimation, 6);
			GunExt.AddProjectileModuleFrom(gun, (Gun)ETGMod.Databases.Items[335], true, false);
			gun.DefaultModule.ammoCost = 1;
			gun.DefaultModule.angleVariance = 0f;
			gun.DefaultModule.shootStyle = ProjectileModule.ShootStyle.SemiAutomatic;
			gun.MovesPlayerForwardOnChargeFire = true;
			gun.DefaultModule.sequenceStyle = ProjectileModule.ProjectileSequenceStyle.Random;
			gun.reloadTime = 1f;
			gun.DefaultModule.preventFiringDuringCharge = true;
			gun.DefaultModule.cooldownTime = 1f;
			gun.DefaultModule.numberOfShotsInClip = 1;
			gun.muzzleFlashEffects = null;
			gun.SetBaseMaxAmmo(50);
			gun.quality = PickupObject.ItemQuality.A;
			gun.gunHandedness = GunHandedness.HiddenOneHanded;
			gun.encounterTrackable.EncounterGuid = "PirateShip";
			Projectile projectile = UnityEngine.Object.Instantiate<Projectile>(((Gun)ETGMod.Databases.Items[327]).DefaultModule.chargeProjectiles[0].Projectile);
			projectile.sprite.renderer.enabled = false;
			projectile.baseData.speed *= 1.8f;
			projectile.baseData.damage = 100f;
			Destroy(projectile.GetComponent<BounceProjModifier>());
			Destroy(projectile.GetComponent<HomingModifier>());
			gun.DefaultModule.projectiles[0] = projectile;
			projectile.gameObject.SetActive(false);
			projectile.gameObject.AddComponent<PirateParrot>();
			FakePrefab.MarkAsFakePrefab(projectile.gameObject);
			UnityEngine.Object.DontDestroyOnLoad(projectile);
			ETGMod.Databases.Items.Add(gun, null, "ANY");
			gun.AddToSubShop(ItemBuilder.ShopType.Trorc, 1f);
			gun.AddToSubShop(ItemBuilder.ShopType.Cursula, 1f);
			gun.AddToSubShop(ItemBuilder.ShopType.Goopton, 1f);
		}

		public override void PostProcessProjectile(Projectile projectile)
		{
			base.PostProcessProjectile(projectile);
		}

		public override void OnPostFired(PlayerController player, Gun gun)
		{
			gun.PreventNormalFireAudio = true;
			AkSoundEngine.PostEvent("Play_WPN_pillow_reload_01", base.gameObject);
		}

		protected void Update()
		{
			bool flag = this.gun.CurrentOwner;
			if (flag)
			{
				if (!gun.PreventNormalFireAudio)
				{
					this.gun.PreventNormalFireAudio = true;
				}
				bool flag4 = !this.gun.IsReloading && !this.HasReloaded;
				if (flag4)
				{
					this.HasReloaded = true;
				}
			}
		}

		public override void OnReloadPressed(PlayerController player, Gun gun, bool bSOMETHING)
		{
			bool flag = gun.IsReloading && this.HasReloaded;
			if (flag)
			{
				this.HasReloaded = false;
				AkSoundEngine.PostEvent("Stop_WPN_All", base.gameObject);
				if (!soundcooldown)
				{
					soundcooldown = true;
					GameManager.Instance.StartCoroutine(Soundcoodown());
					AkSoundEngine.PostEvent("Play_Parrot", base.gameObject);
				}
				base.OnReloadPressed(player, gun, bSOMETHING);
			}
		}

		private IEnumerator Soundcoodown()
		{
			yield return new WaitForSeconds(UnityEngine.Random.Range(40, 60));
			soundcooldown = false;
			yield break;
		}

		private bool HasReloaded;
		private bool soundcooldown;
	}

	internal class PirateParrot : MonoBehaviour
	{
		private static bool m_isDashing;
		private List<AIActor> EnemiesPassedBy;
		private float dashDistance;
		public static GameObject Prefab;
		public static GameObject SplashyPrefab;
		private float dashSpeed = 10f;
		private Gun gun;
		public static GameObject ParrotPrefab;
		public GameObject ExtantParrot;
		private static GameObject CratePrefab;
		private bool TeaCooldown;
		private bool CannonCooldown;
		public RadialBurstInterface RadialBurstSettings;
		private bool doorcollide;
		private DeadlyDeadlyGoopManager DarkGoopDefinition;
		private GameObject TelefragVFXPrefab;
		private List<AIActor> Swallowed;
		private static bool secondaryplayer_isDashing;

		private void Start()
		{
			base.gameObject.GetComponent<Projectile>().OnDestruction += ProcessProjectile;
			if(SplashyPrefab == null)
			{                                           //
				BuildSplashyPrefab();                  //
			}                                         //
			if (Prefab == null)                      //
			{                                       //
				BuildShipPrefab();                 //
			}                                     //
			if (ParrotPrefab == null)            //            these create all the gameobjects for handling the various vfx
			{                                   //
				BuildParrotPrefab();           //
			}                                 //
			if (CratePrefab == null)         //
			{                               //
				BuildCratePrefab();        //
			}                             //
			ExtantParrot = UnityEngine.Object.Instantiate(ParrotPrefab, base.gameObject.transform);   //this attaches the parrot gameobject to our invisible projectile, there's probably a way to actually animate it but it's just *so much easier* to do this instead
			PlayerController user = base.gameObject.GetComponent<Projectile>().Owner as PlayerController;
			gun = user.CurrentGun;
			ExtantParrot.GetComponent<tk2dSpriteAnimator>().Play("idle");                      // this tells the spriteanimator component of our parrot gameobject to play its animation
			if (user.CurrentGun.CurrentAngle > 90f || user.CurrentGun.CurrentAngle < -90f)     // this flips the sprite the right way if the player is firing leftwards as the parrot gameobject inherits the rotation from the projectile and an object rotated past -90/90* is naturally upside down
			{
				ExtantParrot.GetComponent<tk2dSprite>().sprite.FlipY = true;
			}

		}
		public static void BuildShipPrefab() // this makes the ship
		{

			GameObject gameObject = SpriteBuilder.SpriteFromResource("HunterPirateMod/Resources/pirateshit1", null);   ///standard spritebuilder stuff, creates a gameobject with a sprite
			UnityEngine.Object.DontDestroyOnLoad(gameObject);
			FakePrefab.MarkAsFakePrefab(gameObject);
			gameObject.SetActive(false);
			Prefab = gameObject;



		}
		public static void BuildSplashyPrefab() // this creates the splash vfx prefab, the base sprite for this doesn't actually matter as this will be playing our animation anyway 
		{

			GameObject gameObject = SpriteBuilder.SpriteFromResource("HunterPirateMod/Resources/pirateshit1", null);
			gameObject.AddAnimation("idle", "HunterPirateMod/Resources/Splashy", 15, NPCBuilder.AnimationType.Idle, DirectionalAnimation.DirectionType.None, DirectionalAnimation.FlipType.None, tk2dSpriteAnimationClip.WrapMode.Once);  /// this adds an animation to a gameobject, it's basically the same as the normal version in gungeonapi but mines adds warpmode as a variable 
			UnityEngine.Object.DontDestroyOnLoad(gameObject);
			FakePrefab.MarkAsFakePrefab(gameObject);
			gameObject.SetActive(false);
			SplashyPrefab = gameObject;



		}
		public static void BuildParrotPrefab() // this creates the projectile vfx, same deal as the splash vfx, except the animation is set to loop.
		{

			GameObject gameObject = SpriteBuilder.SpriteFromResource("HunterPirateMod/Resources/pirateshit1", null);
			gameObject.AddAnimation("idle", "HunterPirateMod/Resources/ParrotProjectile", 15, NPCBuilder.AnimationType.Idle, DirectionalAnimation.DirectionType.None, DirectionalAnimation.FlipType.None, tk2dSpriteAnimationClip.WrapMode.Loop);
			UnityEngine.Object.DontDestroyOnLoad(gameObject);
			FakePrefab.MarkAsFakePrefab(gameObject);
			gameObject.SetActive(false);
			ParrotPrefab = gameObject;

		}
		public static void BuildCratePrefab()  //this builds the little crates the tea party synergy spawns, same deal as the rest
		{

			GameObject gameObject = SpriteBuilder.SpriteFromResource("HunterPirateMod/Resources/pirateshit1", null);
			gameObject.AddAnimation("idle", "HunterPirateMod/Resources/teacrate", 5, NPCBuilder.AnimationType.Idle, DirectionalAnimation.DirectionType.None, DirectionalAnimation.FlipType.None, tk2dSpriteAnimationClip.WrapMode.Loop);
			UnityEngine.Object.DontDestroyOnLoad(gameObject);
			gameObject.AddComponent<TeaCrateBehaviour>(); //this attaches a component to the crates that handles their behaviour, as with the projectile doing it this way ensures each crate works independently so we don't need to keep track of them  
			FakePrefab.MarkAsFakePrefab(gameObject);
			gameObject.SetActive(false);
			CratePrefab = gameObject;

		}
		public void ProcessProjectile(Projectile projectile)
		{
			try {
				if (base.gameObject.GetComponent<Projectile>().Owner != null && base.gameObject.GetComponent<Projectile>().Owner.sprite != null)
				{
					PlayerController user = base.gameObject.GetComponent<Projectile>().Owner as PlayerController;                            //
					AssetBundle assetBundle = ResourceManager.LoadAssetBundle("shared_auto_001");                                           //   these lines grab the basic water goop from the basegame assets
					GoopDefinition goopDefinition = assetBundle.LoadAsset<GoopDefinition>("assets/data/goops/water goop.asset");           //
					DeadlyDeadlyGoopManager goopManagerForGoopType = DeadlyDeadlyGoopManager.GetGoopManagerForGoopType(goopDefinition);   //
					gun = user.CurrentGun;
					PlayerController playerController = user;
					Vector2 result = projectile.specRigidbody.UnitCenter;
					if (user.PlayerHasActiveSynergy("Kaliber's Trench"))
					{
						if (Swallowed == null)
						{ Swallowed = new List<AIActor>(); }
						goopDefinition = UnityEngine.GameObject.Instantiate(goopDefinition);                      // instantiates goop for the kaliber's trench synergy so the changes to it will not reflect back to the base water goop
						goopDefinition.baseColor32 = new Color32(30, 0, 255, 255);
						goopDefinition.lifespan *= 2f;
						HunterModule.OnGoopTouched += ReleaseTheKracken;                                          // subscribes a method to a custom event that gets called whenever a gameactor (enemies + players basically) physically touches any goop 
						DarkGoopDefinition = DeadlyDeadlyGoopManager.GetGoopManagerForGoopType(goopDefinition);
						goopManagerForGoopType = DarkGoopDefinition;                                               // defines a field with our now custom goop, so we can later identify it 

					}
					goopManagerForGoopType.TimedAddGoopLine(user.CenterPosition, result, 3.5f, 0.5f);
					this.dashDistance = Vector2.Distance(result, playerController.sprite.WorldCenter) - 1.5f;
					this.Effect(user, result);
				}
			}catch (Exception e)
				{ HunterModule.Log(e.ToString() + " " + e.Message + " " + e.Source + " " + e.TargetSite.ToString());}
		}

		private void ReleaseTheKracken(DeadlyDeadlyGoopManager arg1, GameActor arg2, IntVector2 arg3)
		{
			if (arg1 == DarkGoopDefinition)                 // checks if the goop the player is touching is our custom goop, we need to do this becouse the event triggers any time any gameactor touches any goop
			{
				if (arg2 is AIActor && arg2 != null && !arg2.healthHaver.IsBoss && !Swallowed.Contains(arg2 as AIActor))     
				{
					Swallowed.Add(arg2 as AIActor);
					Execute(arg2 as AIActor);
				}
			}
		}
		private void Execute(AIActor ai)
		{
			if (ai.healthHaver != null)
			{
				ETGMod.StartGlobalCoroutine(Tentacler(ai));    // in unity, coroutines (aka the IEnumerator things) automatically stop when the gameobject they're being called from is destroyed, since we want literally all of this happen after the projectile impacts something, that obviously would not to, ETGMod.StartGlobalCoroutine avoids that issue *somehow*  
			}
		}
		private IEnumerator Tentacler(AIActor ai)         // this spawns the yellow chamber tentacle vfx
		{
			yield return new WaitForSeconds(1f);
			ai.behaviorSpeculator.Stun(1f);
			this.TelefragVFXPrefab = (GameObject)ResourceCache.Acquire("Global VFX/VFX_Tentacleport");
			GameObject gameObject = UnityEngine.Object.Instantiate<GameObject>(this.TelefragVFXPrefab);
			gameObject.GetComponent<tk2dBaseSprite>().PlaceAtLocalPositionByAnchor(ai.specRigidbody.UnitCenter, tk2dBaseSprite.Anchor.LowerCenter);
			gameObject.transform.position = gameObject.transform.position.Quantize(0.0625f);
			gameObject.GetComponent<tk2dBaseSprite>().UpdateZDepth();
			yield return new WaitForSeconds(1f);
			ai.EraseFromExistenceWithRewards(false);
			yield break;
		}
		private void Effect(PlayerController user, Vector2 target)
		{
			if (user = GameManager.Instance.PrimaryPlayer)
			{
				if (!PirateParrot.m_isDashing)                                                         //this checks if the player is already sailing, the use of a static bool makes it so all projectile instances refer to the same value
				{
					Vector2 vector = target - user.specRigidbody.UnitCenter;
					bool flag = vector.x == 0f && vector.y == 0f;
					if (!flag)
					{
						ETGMod.StartGlobalCoroutine(HandleDash(user, vector));
					}

				}
			}
			if (GameManager.Instance.SecondaryPlayer && user == GameManager.Instance.SecondaryPlayer)   //this checks if the player is player 2 and does the same, becouse co-op exists
			{
				if (!PirateParrot.secondaryplayer_isDashing)
				{
					Vector2 vector = target - user.specRigidbody.UnitCenter;
					bool flag = vector.x == 0f && vector.y == 0f;
					if (!flag)
					{
						ETGMod.StartGlobalCoroutine(HandleDash(user, vector));
					}

				}
			}
		}


		public IEnumerator HandleDash(PlayerController user, Vector2 dashDirection)
		{
			doorcollide = false;
			if (user == GameManager.Instance.PrimaryPlayer)
			{
				PirateParrot.m_isDashing = true;
			}
			if (GameManager.Instance.SecondaryPlayer && user == GameManager.Instance.SecondaryPlayer)
			{
				PirateParrot.secondaryplayer_isDashing = true;
		    }
			EnemiesPassedBy = new List<AIActor>();
			String strin = "pirateship" + Guid.NewGuid().ToString();                       //  this nullifies fall damage, and saves the id string so we can remove the modifier later  
			user.ImmuneToPits.SetOverride(strin, true, null);                             //
			GameObject ship = UnityEngine.Object.Instantiate(Prefab, user.transform);                       //this spawns the ship gameobject as a child of the player
			ship.GetComponent<tk2dSprite>().renderer.enabled = false;                                       //this makes the ship invisible
			GameObject SplashyPrefab = UnityEngine.Object.Instantiate(PirateParrot.SplashyPrefab, ship.transform); //this spawns the splash vfx gameobject as a child of the ship
			if (gun.CurrentAngle > 90f || gun.CurrentAngle < -90f)                                           //flips the sprites, same deal with the parrot, and manages the offsets
			{
				ship.GetComponent<tk2dSprite>().sprite.FlipX = true;
				ship.transform.localPosition += new Vector3(3.5f, -0.8f, 0f);
				SplashyPrefab.transform.localPosition += new Vector3(-3.5f, -1.5f, 0f);

			}
			else
			{
				ship.GetComponent<tk2dSprite>().sprite.FlipX = false;
				ship.transform.localPosition -= new Vector3(2.5f, 0.8f, 0f);
				SplashyPrefab.transform.localPosition -= new Vector3(1f, 1.5f, 0f);

			}
			SplashyPrefab.GetComponent<tk2dSpriteAnimator>().Play("idle");                                //this tells the splash vfx gameobject to play its animation
			int frames = 0;
			while (SplashyPrefab.GetComponent<tk2dSpriteAnimator>().IsPlaying("idle") && frames < 7)       //this temporarily delays the rest of the code in the routine while the animation is playing
			{
				frames += 1;
				if (frames == 7)
				{
					ship.GetComponent<tk2dSprite>().renderer.enabled = true;         //this makes the ship visible
				}
				yield return null;
			}
			GameManager.Instance.StartCoroutine(SeaSounds());
			ship.GetComponent<tk2dSprite>().sprite.HeightOffGround = user.sprite.HeightOffGround - 5f;          // this manages the depth of the ship
			ship.GetComponent<tk2dSprite>().sprite.UpdateZDepth();                                              // this makes it so the last line actually does annything
			Vector2 startPosition = user.sprite.WorldCenter;
			user.SetInputOverride("pirateship");                                                                 //this locks the player's controls
			user.healthHaver.IsVulnerable = false;                                                               // makes the player invulnerable
			user.FallingProhibited = true;                                                                       // prevents them from falling into pits
			PixelCollider playerHitbox = user.specRigidbody.HitboxPixelCollider;                                              //
			playerHitbox.CollisionLayerCollidableOverride |= CollisionMask.LayerToMask(CollisionLayer.EnemyHitBox);          // makes them go through enemies
			SpeculativeRigidbody specRigidbody = user.specRigidbody;
			specRigidbody.OnPreRigidbodyCollision  += this.PreCollision;                                       
			specRigidbody.OnPreTileCollision += this.PreTileCollision;
			float duration = Mathf.Clamp((this.dashDistance / this.dashSpeed), 0.1f,10f);                                   // this determines how long the player sails for, up to a maximum of 10 which is a time unit though i have no idea to how many seconds it actually equates to.
			float elapsed = 0f;
			while (elapsed < duration && !doorcollide)
			{
				
				if (!SplashyPrefab.GetComponent<tk2dSpriteAnimator>().IsPlaying("idle"))                          // 
				{                                                                                                //
					SplashyPrefab.GetComponent<tk2dSprite>().renderer.enabled = false;                          //   makes the splashes invisible if the animation is finished
				}                                                                                              //
				if (user.CurrentRoom != null && user.CurrentRoom.GetActiveEnemies(RoomHandler.ActiveEnemyType.All) != null && user.CurrentRoom.GetActiveEnemies(RoomHandler.ActiveEnemyType.All).Count > 0)                //
				{                                                                                                                                                                                                         //
					foreach (AIActor actor in user.CurrentRoom.GetActiveEnemies(RoomHandler.ActiveEnemyType.All))                                                                                                        //
					{                                                                                                                                                                                                   //      Adds all the enemies in a radius to a list, if they're not in that list already (very important as this gets called once per frame by the loop)
						if (actor != null && Vector2.Distance(actor.CenterPosition, user.CenterPosition) < 4f && !EnemiesPassedBy.Contains(actor))                                                                     //
						{                                                                                                                                                                                             //
							EnemiesPassedBy.Add(actor);                                                                                                                                                              //
						}


					}
				}
				Synergyhandler(user, dashDirection);
				user.healthHaver.IsVulnerable = false;
				elapsed += BraveTime.DeltaTime;
				float adjSpeed = Mathf.Min(this.dashSpeed, this.dashDistance / BraveTime.DeltaTime);                        // changes the speed based on the distance and the time it needs to take to get there.
				user.specRigidbody.Velocity = dashDirection.normalized * adjSpeed;                                          // the bit that actually makes the player move
				yield return null;
			}
			SplashyPrefab.GetComponent<tk2dSprite>().renderer.enabled = true;
			ship.GetComponent<tk2dSprite>().renderer.enabled = false;
			SplashyPrefab.GetComponent<tk2dSprite>().transform.parent = null;
			SplashyPrefab.GetComponent<tk2dSpriteAnimator>().Play("idle");
			Projectile Boomprojectile = (ItemAPI.FakePrefab.Clone(((Gun)ETGMod.Databases.Items[129]).DefaultModule.projectiles[0].gameObject)).GetComponent<Projectile>();   // instantiates a projectile, in this case the com4ndo's
			ExplosiveModifier boomer = Boomprojectile.GetComponent<ExplosiveModifier>();                                                           //
			boomer.explosionData.doScreenShake = false;                                                                                           //
			boomer.explosionData.damageRadius = 4.5f;                                                                                            //
			boomer.explosionData.doExplosionRing = true;                                                                                        //
			boomer.explosionData.damageToPlayer = 0f;                                                                                          //
			boomer.explosionData.force = 90f;                                                                                                 //           does a custom explosion based on the instantiated projectile's
			boomer.explosionData.preventPlayerForce = true;                                                                                  //
			boomer.explosionData.pushRadius = boomer.explosionData.damageRadius;                                                            //
			boomer.explosionData.damage = 35f * user.stats.GetStatValue(PlayerStats.StatType.Damage);                                      //
			boomer.IgnoreQueues = true;                                                                                                   //
			Exploder.Explode(user.transform.position, boomer.explosionData, Vector2.zero, null, false, CoreDamageTypes.None, false);     //
			playerHitbox.CollisionLayerCollidableOverride &= ~CollisionMask.LayerToMask(CollisionLayer.EnemyHitBox);   // makes the player collide with enemies again
			SpeculativeRigidbody specRigidbody2 = user.specRigidbody;
			specRigidbody.OnPreRigidbodyCollision -= this.PreCollision;
			specRigidbody.OnPreTileCollision -= this.PreTileCollision;
			user.FallingProhibited = false;
			user.ClearInputOverride("pirateship");                           // gives the player back controls
			yield return  new WaitForSeconds(0.4f);
			Destroy(SplashyPrefab);                                          //gets rid of both vfx gameobjects
			Destroy(ship);                                                  //
			user.healthHaver.IsVulnerable = true;
			if (user == GameManager.Instance.PrimaryPlayer)
			{
				PirateParrot.m_isDashing = false;
			}
			if (GameManager.Instance.SecondaryPlayer && user == GameManager.Instance.SecondaryPlayer)
			{
				PirateParrot.secondaryplayer_isDashing = false;
			}
			GameManager.Instance.StartCoroutine(Eatem(user));
			yield return new WaitForSeconds(2.3f);
			user.ImmuneToPits.RemoveOverride(strin);                  // removes the immunity to fall damage
			yield break;
		}

		private IEnumerator SeaSounds()  //this plays the wave sounds at a regular interval while the player is sailing
		{
			while(PirateParrot.m_isDashing)
			{
				AkSoundEngine.PostEvent("Play_Wave", base.gameObject);
				yield return 0.009f;
			}
			while (PirateParrot.secondaryplayer_isDashing)
			{
				AkSoundEngine.PostEvent("Play_Wave", base.gameObject);
				yield return 0.009f;
			}
			yield break;
		}

		private void PreTileCollision(SpeculativeRigidbody myRigidbody, PixelCollider myPixelCollider, PhysicsEngine.Tile tile, PixelCollider tilePixelCollider)   // makes sure the player bounces off walls and doesn't get stuck 
		{
			doorcollide = true;
			PhysicsEngine.SkipCollision = false;
			myRigidbody.knockbackDoer.ApplyKnockback(myRigidbody.Velocity.Rotate(180f), 30f);
		}

		private IEnumerator Eatem(PlayerController user)         // goes through the list of enemies the player has sailed by one by one and spawns an invisible super homing air tank shark after them 
		{
			int i = 0;
			while (EnemiesPassedBy.Count > 0)
			{
				if (EnemiesPassedBy[i] != null)
				{
					Projectile projectile2 = ((Gun)global::ETGMod.Databases.Items[359]).DefaultModule.chargeProjectiles[0].Projectile;
					GameObject gameObject = SpawnManager.SpawnProjectile(projectile2.gameObject, EnemiesPassedBy[i].specRigidbody.UnitCenter, Quaternion.identity, true);
					Projectile component = gameObject.GetComponent<Projectile>();
					if (component != null)
					{
						component.sprite.renderer.enabled = false;
						HomingModifier homing = component.gameObject.AddComponent<HomingModifier>();
						homing.AngularVelocity = 10000000f;
						homing.HomingRadius = 3f;
						component.Owner = user;
						component.baseData.damage *= 2f;
						component.Shooter = user.specRigidbody;
						component.baseData.speed = 10f;
						component.ignoreDamageCaps = false;
					}
					yield return new WaitForSeconds(0.35f);
				}
				i += 1;
				yield return null;

			}
			EnemiesPassedBy.Clear();
			yield break;
		}

		private void Synergyhandler(PlayerController user, Vector2 dir)
		{
			if (user.PlayerHasActiveSynergy("Sons Of Liberty") && !TeaCooldown)
			{
				TeaCooldown = true;
				GameManager.Instance.StartCoroutine(TeaCold());
				UnityEngine.Object.Instantiate(CratePrefab, new Vector2(user.CenterPosition.x - dir.normalized.x, user.CenterPosition.y + UnityEngine.Random.Range(-1f, 1f)), Quaternion.identity);       // actually spawns the crates for the tea party synergy 
			}
			if (user.PlayerHasActiveSynergy("Cannon Crew") && !CannonCooldown)
			{
				CannonCooldown = true;
				GameManager.Instance.StartCoroutine(CannonFire());
				RadialBurstSettings.SpiralWaves = false;                                                                 //
				RadialBurstSettings.NumberWaves = 1;                                                                    //
				RadialBurstSettings.MaxToSpawnPerWave = 6;                                                             //
				Projectile projectile = ((Gun)ETGMod.Databases.Items[675]).DefaultModule.projectiles[0];              //   calls the class that handles radial burst of projectiles for items like ring of triggers and tells it to do a burst with specific variables
				RadialBurstSettings.ProjectileInterface.UseCurrentGunProjectile = false;                             //
				RadialBurstSettings.ProjectileInterface.SpecifiedProjectile = projectile;                           //
				RadialBurstSettings.DoBurst(user, user.sprite.WorldBottomCenter, null);                            //
			}
		}

		private IEnumerator TeaCold()
		{
			yield return new WaitForSeconds(0.15f);
			TeaCooldown = false;
			yield break;
		}
		private IEnumerator CannonFire()
		{
			yield return new WaitForSeconds(0.6f);
			CannonCooldown = false;
			yield break;
		}

		private void PreCollision(SpeculativeRigidbody myRigidbody, PixelCollider myPixelCollider, SpeculativeRigidbody otherRigidbody, PixelCollider otherPixelCollider)   // makes sure tables will be broken when sailed through as well as that the player doesn't get stuck in locked doors
		{
			PhysicsEngine.SkipCollision = true;
			if (otherRigidbody.majorBreakable != null)
			{
				PhysicsEngine.SkipCollision = true;
				otherRigidbody.majorBreakable.ApplyDamage(100f, Vector2.one, false, false, false);   // does the table breaking
			}
			if (otherRigidbody.gameObject.GetComponent<DungeonDoorController>() != null || otherRigidbody.gameObject.GetComponentInParent<DungeonDoorController>() != null || otherRigidbody.gameObject.GetComponentInChildren<DungeonDoorController>() != null)
			{
				PhysicsEngine.SkipCollision = false;
				doorcollide = true;
				myRigidbody.knockbackDoer.ApplyKnockback(myRigidbody.Velocity.Rotate(180f), 30f);        //does the not getting stuck in locked doors
			}

		}

		private class TeaCrateBehaviour : MonoBehaviour
		{
			private int CrateHits;
			private tk2dSprite Sprite;
			public RadialBurstInterface RadialBurstSettings;

			private void Start()
			{
				SetUpRigidbody(base.gameObject, new IntVector2(19, 19)).OnPreRigidbodyCollision += CrateDamaged;  // gives the crate a specrigidbody, as well as subscirbing a method to the event called when the rigidbody collides with something
				Sprite = base.gameObject.GetComponent<tk2dSprite>();
				base.gameObject.GetComponent<tk2dSpriteAnimator>().Play("idle");
				GameManager.Instance.StartCoroutine(HandleCrate());
				CrateHits = 0;
			
			
			}

			private void CrateDamaged(SpeculativeRigidbody myRigidbody, PixelCollider myPixelCollider, SpeculativeRigidbody otherRigidbody, PixelCollider otherPixelCollider)  //keeps track of how many times the crate has been hit
			{
				CrateHits += 1;
			}

			private IEnumerator HandleCrate()
			{

				while(base.gameObject != null && CrateHits < 2)          //keeps water around the crate while it's "alive"
				{
					AssetBundle assetBundle = ResourceManager.LoadAssetBundle("shared_auto_001");
					GoopDefinition goopDefinition = assetBundle.LoadAsset<GoopDefinition>("assets/data/goops/water goop.asset");
					DeadlyDeadlyGoopManager goopManagerForGoopType = DeadlyDeadlyGoopManager.GetGoopManagerForGoopType(goopDefinition);
					goopManagerForGoopType.TimedAddGoopCircle(Sprite.WorldCenter, 1.75f);
					yield return new WaitForSeconds(0.05f);
				}
				Destroy(base.gameObject.GetComponent<SpeculativeRigidbody>());  // destroys the crate's rigidbody so the bust of leaves goes through
				RadialBurstSettings.SpiralWaves = false;
				RadialBurstSettings.NumberWaves = 2;
				Projectile projectile = ((Gun)global::ETGMod.Databases.Items[620]).DefaultModule.chargeProjectiles[0].Projectile;
				RadialBurstSettings.ProjectileInterface.UseCurrentGunProjectile = false;
				RadialBurstSettings.ProjectileInterface.SpecifiedProjectile = projectile;
				RadialBurstSettings.CustomPostProcessProjectile += TeaLeaves; 
				RadialBurstSettings.DoBurst(GameManager.Instance.PrimaryPlayer, Sprite.WorldCenter, null);
				Projectile Boomprojectile = (ItemAPI.FakePrefab.Clone(((Gun)ETGMod.Databases.Items[129]).DefaultModule.projectiles[0].gameObject)).GetComponent<Projectile>();
				ExplosiveModifier boomer = Boomprojectile.GetComponent<ExplosiveModifier>();
				boomer.explosionData.doScreenShake = false;
				boomer.explosionData.damageRadius = 4.5f;
				boomer.explosionData.doExplosionRing = true;
				boomer.explosionData.damageToPlayer = 0f;
				boomer.explosionData.force = 90f;
				boomer.explosionData.preventPlayerForce = true;
				boomer.explosionData.pushRadius = boomer.explosionData.damageRadius;
				boomer.explosionData.damage = 35f;
				boomer.IgnoreQueues = true;
				Exploder.Explode(Sprite.WorldCenter, boomer.explosionData, Vector2.zero, null, false, CoreDamageTypes.None, false);
				Destroy(base.gameObject);      //destroys the crate
				yield break;
			}

			private void TeaLeaves(Projectile obj)
			{
				obj.baseData.range = 6 * UnityEngine.Random.Range(0.5f, 1f);
				obj.baseData.speed *= UnityEngine.Random.Range(0.5f, 1f);
				Color color = Color.HSVToRGB((UnityEngine.Random.Range(30f, 90f) / 360), 1, 3f);
				obj.AdjustPlayerProjectileTint(color, 100, 0); 
			}

			public static SpeculativeRigidbody SetUpRigidbody(GameObject Ob, IntVector2 dimensions)
			{
					var body = Ob.GetOrAddComponent<SpeculativeRigidbody>();
					PixelCollider collider = new PixelCollider();
				    collider.ColliderGenerationMode = PixelCollider.PixelColliderGeneration.Manual;
				    collider.CollisionLayer = CollisionLayer.BulletBlocker;
				    collider.ManualWidth = dimensions.x;
				    collider.ManualHeight = dimensions.y;
				    body.PixelColliders = new List<PixelCollider>();
				    body.PixelColliders.Add(collider);
					return body;
			}
	
		}
	}
}
